@@ -1,14 +1,102 @@
-"""Utility functions used by various web helpers
+"""Utility functions used by various web helpers.
 
-This module is such a mess that no new helpers should be added to it.
+This module contains support functions used by other helpers, and functions for
+URL manipulation. Most of these helpers predate the 0.6 reorganization; they
+would have been put in other subpackages if they have been created later.
 """
 import cgi
 import copy
 import sys
 import urllib
+import urlparse
 from UserDict import DictMixin
 from xml.sax.saxutils import XMLGenerator
 
+try:
+    from urlparse import parse_qs
+except ImportError:   # Python < 2.6
+    from cgi import parse_qs
+
+
+def update_params(_url, _debug=False, **params):
+    """Update query parameters in a URL.
+
+    ``_url`` is any URL, with or without a query string.
+
+    ``\*\*params`` are query parameters to add or replace. Each value may be a
+    string, a list of strings, or None. Passing a list generates multiple
+    values for the same parameter. Passing None deletes the corresponding
+    parameter if present.
+
+    Return the new URL.
+
+    *Debug mode:* if a pseudo-parameter ``_debug=True`` is passed,
+    return a tuple: ``[0]`` is the URL without query string or fragment,
+    ``[1]`` is the final query parameters as a dict, and ``[2]`` is the
+    fragment part of the original URL or the empty string.
+
+    Usage:
+
+    >>> update_params("foo", new1="NEW1")
+    'foo?new1=NEW1'
+    >>> update_params("foo?p=1", p="2")
+    'foo?p=2'
+    >>> update_params("foo?p=1", p=None)
+    'foo'
+    >>> update_params("http://example.com/foo?new1=OLD1#myfrag", new1="NEW1")
+    'http://example.com/foo?new1=NEW1#myfrag'
+    >>> update_params("http://example.com/foo?new1=OLD1#myfrag", new1="NEW1", _debug=True)
+    ('http://example.com/foo', {'new1': 'NEW1'}, 'myfrag')
+    >>> update_params("http://www.mau.de?foo=2", brrr=3)
+    'http://www.mau.de?foo=2&brrr=3'
+    >>> update_params("http://www.mau.de?foo=A&foo=B", foo=["C", "D"])
+    'http://www.mau.de?foo=C&foo=D'
+
+    """
+    url, fragment = urlparse.urldefrag(_url)
+    if "?" in url:
+        url, qs = url.split("?", 1)
+        query = parse_qs(qs)
+    else:
+        query = {}
+    for key, value in params.iteritems():
+        if value is not None:
+            query[key] = value
+        elif key in query:
+            del query[key]
+    if _debug:
+        return url, query, fragment
+    qs = urllib.urlencode(query, True)
+    if qs:
+        qs = "?" + qs
+    if fragment:
+        fragment = "#" + fragment
+    return "%s%s%s" % (url, qs, fragment)
+
+def cgi_escape(s, quote=False):
+    """Replace special characters '&', '<' and '>' by SGML entities.
+
+    This is a slightly more efficient version of the cgi.escape by
+    using 'in' membership to test if the replace is needed.
+
+    This function returns a plain string. Programs using the HTML builder
+    should call ``webhelpers.html.builder.escape()`` instead of this to prevent
+    double-escaping.
+
+    Changed in WebHelpers 1.2: escape single-quote as well as double-quote.
+
+    """
+    if '&' in s:
+        s = s.replace("&", "&amp;") # Must be done first!
+    if '<' in s:
+        s = s.replace("<", "&lt;")
+    if '>' in s:
+        s = s.replace(">", "&gt;")
+    if quote:
+        s = s.replace('"', "&quot;")
+        s = s.replace("'", "&apos;")
+    return s
+
 def html_escape(s):
     """HTML-escape a string or object.
     
@@ -19,6 +107,9 @@ def html_escape(s):
     
     None is treated specially, and returns the empty string.
     
+    This function returns a plain string. Programs using the HTML builder
+    should wrap the result in ``literal()`` to prevent double-escaping.
+
     """
     if s is None:
         return ''
@@ -27,7 +118,7 @@ def html_escape(s):
             s = unicode(s)
         else:
             s = str(s)
-    s = cgi.escape(s, True)
+    s = cgi_escape(s, True)
     if isinstance(s, unicode):
         s = s.encode('ascii', 'xmlcharrefreplace')
     return s
@@ -44,8 +135,10 @@ def iri_to_uri(iri):
     simplify things a little from the full method.
 
     Returns an ASCII string containing the encoded result.
-    
+
     """
+    # Called by webhelpers.feedgenerator
+    #
     # The list of safe characters here is constructed from the printable ASCII
     # characters that are not explicitly excluded by the list at the end of
     # section 3.1 of RFC 3987.
@@ -56,7 +149,12 @@ def iri_to_uri(iri):
 
 class Partial(object):
     
-    """Partial object, which will be in Python 2.5"""
+    """
+    A partial function object.
+
+    Equivalent to functools.partial, which was introduced in Python 2.5.
+
+    """
     
     def __init__(*args, **kw):
         self = args[0]
@@ -71,8 +169,14 @@ class Partial(object):
         return self.fn(*(self.args + args), **d)
 
 class SimplerXMLGenerator(XMLGenerator):
-    def addQuickElement(self, name, contents=None, attrs={}):
+    """A subclass of Python's SAX XMLGenerator."""
+
+    # Used by webhelpers.feedgenerator
+
+    def addQuickElement(self, name, contents=None, attrs=None):
         """Add an element with no children."""
+        if attrs is None:
+            attrs = {}
         self.startElement(name, attrs)
         if contents is not None:
             self.characters(contents)
@@ -86,7 +190,7 @@ class UnicodeMultiDict(DictMixin):
     Decoding is not applied to assigned values.
 
     The key/value contents are assumed to be ``str``/``strs`` or
-    ``str``/``FieldStorages`` (as is returned by the paste.request.parse_
+    ``str``/``FieldStorages`` (as is returned by the :func:`paste.request.parse`
     functions).
 
     Can optionally also decode keys when the ``decode_keys`` argument is
